@@ -23,7 +23,7 @@ WWW::YoutubeViewer - A very easy interface to YouTube.
 
 =cut
 
-our $VERSION = '3.5.5';
+our $VERSION = '3.5.8';
 
 =head1 SYNOPSIS
 
@@ -94,7 +94,7 @@ my %valid_options = (
 
     authentication_file => {valid => [qr/^./], default => undef},
 
-    # No input value alowed
+    # No input value allowed
     feeds_url        => {valid => q[], default => 'https://www.googleapis.com/youtube/v3/'},
     video_info_url   => {valid => q[], default => 'https://www.youtube.com/get_video_info'},
     oauth_url        => {valid => q[], default => 'https://accounts.google.com/o/oauth2/'},
@@ -223,7 +223,7 @@ sub escape_string {
 
 =head2 set_lwp_useragent()
 
-Intializes the LWP::UserAgent module and returns it.
+Initializes the LWP::UserAgent module and returns it.
 
 =cut
 
@@ -324,7 +324,7 @@ sub _auth_lwp_header {
 
 sub _warn_reponse_error {
     my ($resp, $url) = @_;
-    warn sprintf("[%s] Error occured on URL: %s\n", $resp->status_line, $url =~ s/([&?])key=(.*?)&/${1}key=[...]&/r);
+    warn sprintf("[%s] Error occurred on URL: %s\n", $resp->status_line, $url =~ s/([&?])key=(.*?)&/${1}key=[...]&/r);
 }
 
 =head2 lwp_get($url, %opt)
@@ -497,7 +497,15 @@ sub _extract_from_invidious {
 
     my $url = sprintf("https://invidio.us/api/v1/videos/%s?fields=formatStreams,adaptiveFormats", $videoID);
 
-    (my $resp = $self->{lwp}->get($url))->is_success() || return;
+    my $tries = 3;
+    my $resp  = $self->{lwp}->get($url);
+
+    while (not $resp->is_success() and $resp->status_line() =~ /read timeout/i and --$tries >= 0) {
+        $resp = $self->{lwp}->get($url);
+    }
+
+    $resp->is_success() || return;
+
     my $json = $resp->decoded_content()        // return;
     my $ref  = $self->parse_json_string($json) // return;
 
@@ -630,7 +638,7 @@ sub _group_keys_with_values {
     return @hashes;
 }
 
-sub _extract_streaming_urls {
+sub _old_extract_streaming_urls {
     my ($self, $info, $videoID) = @_;
 
     my %stream_map = $self->parse_query_string($info->{url_encoded_fmt_stream_map}, multi => 1);
@@ -676,6 +684,69 @@ sub _extract_streaming_urls {
     return @results;
 }
 
+sub _extract_streaming_urls {
+    my ($self, $info, $videoID) = @_;
+
+    if (exists $info->{url_encoded_fmt_stream_map}) {
+        return $self->_old_extract_streaming_urls($info, $videoID);
+    }
+
+    if ($self->get_debug) {
+        say STDERR ":: Using the newer format to extract the streaming URLs...";
+    }
+
+    my $json = $self->parse_json_string($info->{player_response});
+
+    if ($self->get_debug >= 2) {
+        require Data::Dump;
+        Data::Dump::pp($json);
+    }
+
+    my @results;
+    if (exists $json->{streamingData}) {
+        if (exists $json->{streamingData}{adaptiveFormats}) {
+            push @results, @{$json->{streamingData}{adaptiveFormats}};
+        }
+
+        if (exists $json->{streamingData}{formats}) {
+            push @results, @{$json->{streamingData}{formats}};
+        }
+    }
+
+    foreach my $item (@results) {
+
+        if (exists $item->{cipher} and not exists $item->{url}) {
+            my (%data) = $self->parse_query_string($item->{cipher});
+
+            $item->{url} = $data{url} if defined($data{url});
+
+            if (defined($data{s})) {    # unclear how this can be decrypted...
+                require URI::Escape;
+                my $sig = $data{s};
+                $sig = URI::Escape::uri_escape($sig);
+                $item->{url} .= "&sig=$sig";
+            }
+        }
+
+        if (exists $item->{mimeType}) {
+            $item->{type} = $item->{mimeType};
+        }
+    }
+
+    # Cipher streaming URLs are currently unsupported, so let's filter them out.
+    @results = grep { not exists $_->{cipher} } @results;
+
+    # Keep only streams with contentLength > 0.
+    @results = grep { exists($_->{contentLength}) and $_->{contentLength} > 0 } @results;
+
+    if ($self->get_debug) {
+        my $count = scalar(@results);
+        say STDERR ":: Found $count streaming URLs...";
+    }
+
+    return @results;
+}
+
 =head2 get_streaming_urls($videoID)
 
 Returns a list of streaming URLs for a videoID.
@@ -704,7 +775,7 @@ sub get_streaming_urls {
         }
     }
 
-    if ($self->get_debug == 2) {
+    if ($self->get_debug == 3) {
         require Data::Dump;
         Data::Dump::pp(\%info);
         Data::Dump::pp(\@streaming_urls);
@@ -737,6 +808,20 @@ sub get_streaming_urls {
         }
     }
 
+    # Filter out streams with `clen = 0`.
+    @streaming_urls = grep { defined($_->{clen}) ? ($_->{clen} > 0) : 1 } @streaming_urls;
+
+    # Return the YouTube URL when there are no streaming URLs
+    if (!@streaming_urls) {
+        push @streaming_urls,
+          {
+            itag    => 22,
+            quality => "hd720",
+            type    => "video/mp4",
+            url     => "https://www.youtube.com/watch?v=$videoID",
+          };
+    }
+
     return (\@streaming_urls, \@caption_urls, \%info);
 }
 
@@ -787,7 +872,7 @@ sub post_as_json {
     $self->_save('POST', $url, $json_str);
 }
 
-# SOUBROUTINE FACTORY
+# SUBROUTINE FACTORY
 {
     no strict 'refs';
 
