@@ -24,7 +24,7 @@ sub _get_token_oauth_url {
 
 =head2 oauth_refresh_token()
 
-Refresh the access_token using the refresh_token. Returns a JSON string or undef.
+Refresh the access_token using the refresh_token. Returns a HASH ref with the `access_token` or undef.
 
 =cut
 
@@ -191,7 +191,7 @@ sub save_authentication_tokens {
 
 =head1 AUTHOR
 
-Trizen, C<< <trizenx at gmail.com> >>
+Trizen, C<< <echo dHJpemVuQHByb3Rvbm1haWwuY29tCg== | base64 -d> >>
 
 
 =head1 SUPPORT
